#pylint: disable=C0103,R0903

from ansi.sequence import sequence
from ansi.colour.base import Graphic
from ansi.colour.fx import bright

black           = Graphic('40')
red             = Graphic('41')
green           = Graphic('42')
brown           = Graphic('43')
blue            = Graphic('44')
magenta         = Graphic('45')
cyan            = Graphic('46')
grey            = Graphic('47')
gray            = grey                  # US English
default         = Graphic('49')
darkgrey        = bright + black
darkgray        = darkgrey              # US English
brightred       = bright + red
brightgreen     = bright + green
yellow          = bright + brown
brightblue      = bright + blue
brightmagenta   = bright + magenta
brightcyan      = bright + cyan
white           = bright + grey



from ansi.sequence import sequence


ANSI_COLOURS = (
    (0x00, 0x00, 0x00), (0xcd, 0x00, 0x00),
    (0x00, 0xcd, 0x00), (0xcd, 0xcd, 0x00),
    (0x00, 0x00, 0xee), (0xcd, 0x00, 0xcd),
    (0x00, 0xcd, 0xcd), (0xe5, 0xe5, 0xe5),
    (0x7f, 0x7f, 0x7f), (0xff, 0x00, 0x00),
    (0x00, 0xff, 0x00), (0xff, 0xff, 0x00),
    (0x5c, 0x5c, 0xff), (0xff, 0x00, 0xff),
    (0x00, 0xff, 0xff), (0xff, 0xff, 0xff),
)

def rgb_distance(rgb1, rgb2):
    '''
    Calculate the distance between two RGB sequences.
    '''
    return sum(map(lambda c: (c[0] - c[1]) ** 2,
        zip(rgb1, rgb2)))

def rgb_reduce(r, g, b, mode=8):
    '''
    Convert an RGB colour to 8 or 16 colour ANSI graphics.
    '''
    colours = ANSI_COLOURS[:mode]
    matches = [(rgb_distance(c, map(int, [r, g, b])), i)
        for i, c in enumerate(colours)]
    matches.sort()
    return sequence('m')(str(30 + matches[0][1]))

def rgb8(r, g, b):
    '''
    Convert an RGB colour to 8 colour ANSI graphics.
    '''
    return rgb_reduce(r, g, b, 8)

def rgb16(r, g, b):
    '''
    Convert an RGB colour to 16 colour ANSI graphics.
    '''
    return rgb_reduce(r, g, b, 16)

def rgb256(r, g, b):
    '''
    Convert an RGB colour to 256 colour ANSI graphics.
    '''
    grey = False
    poss = True
    step = 2.5

    while poss: # As long as the colour could be grey scale
        if r < step or g < step or b < step:
            grey = r < step and g < step and b < step
            poss = False

        step += 42.5

    if grey:
        colour = 232 + int(float(sum([r, g, b]) / 33.0))
    else:
        colour = sum([16] + [int (6 * float(val) / 256) * mod
            for val, mod in ((r, 36), (g, 6), (b, 1))])

    return sequence('m', fields=3)(38, 5, colour)



#pylint: disable=C0103,R0903

from ansi.colour.base import Graphic


reset           = Graphic('0')
bold            = Graphic('1')
bright          = bold
faint           = Graphic('2')
italic          = Graphic('3')
underline       = Graphic('4')
blink_slow      = Graphic('5')
blink           = Graphic('6')
inverse         = Graphic('7')
conceal         = Graphic('8')
crossed_out     = Graphic('9')
font_reset      = Graphic('10')
normal          = Graphic('22')
blink_off       = Graphic('25')
reveal          = Graphic('28')
